 /*  yaxe.c  Yet Another XCOM Editor    */
 /*  August 30, 1994  */

 #include <stdio.h>
 #include <fcntl.h>
 #include <io.h>
 #include <errno.h>
 #include <string.h>
 #include <sys\types.h>
 #include <sys\stat.h>
 #include <ctype.h>


 #define MAXSOLD 250
 #define YES 1
 #define NO 0

 struct soldier
	{       int rank;
		int base;
		int ship;
		int u1;         /* always -1 */
		int missions;
		int kills;
		int daystoheal;
		int experience;
		char name[26];
		char btu;
		char bhealth;
		char bstamina;
		char breaction;
		char bstr;
		char bfa;
		char bta;
		char brecov;
		char bpsistr;
		char bpsiskill;
		char bbravery;
		char mtu;
		char mhealth;
		char mstamina;
		char mreaction;
		char mstr;
		char mfa;
		char mta;
		char mrecov;
		char mpsistr;
		char armor;
		char mbravery;  /* May be mpsiskill */
		char trng;
		char u3;
		char u4;
		char u5;
	};

void printuse(int);             /*Print proper useage and exit*/
void shipsort(int,int);         /*Sorts soldier lists*/
void sswitch(int,int);          /*Switch two real records*/
void showtroop(int,int,int);    /*Display troop to screen*/

struct soldier troop[MAXSOLD];
int si[MAXSOLD];                /* sorting index by base, ship */
int tu[MAXSOLD],fa[MAXSOLD],rx[MAXSOLD],str[MAXSOLD];
int pri[MAXSOLD], sec[MAXSOLD];

void main( int argc, char *argv[])
{       char *source, *cp, *filenam, prikey, seckey;
	int manual=NO,sort=NO, a, b;
	int pridir=1,secdir=1;
	int hsource,  ch, oldbase, oldship;
	int index,rcount, i, flip, temp;
	long tlong;

	source="GAME_\0              \0";
	filenam="\\soldier.dat\0";

	switch (argc)
	{  case 1:
	     printuse(1);
	     break;       /*Not really necessary because printuse exits*/
	   case 2:
	     if(isdigit(*argv[1]))
	       {  source=strcat(source,argv[1]);
	       }
	       else
	       {  printuse(2);
	       }
	     break;
	   default:     /* argc is at least 3 */
	     if(isdigit(*argv[2]))
	       {  source=strcat(source,argv[2]);
		  /*  Now parse options */
		  cp = argv[1];
		  if (*cp!='/'&&*cp!='-') printuse(3);
		  cp++;
		  for(i=1;i<strlen(argv[1]);i++,cp++)
		    {  switch (*cp)
		       { case 'm':
			 case 'M':
			   manual=YES;
			   break;
			 case 'r':
			 case 'R':
			 case 'p':
			 case 'P':
			 case 'f':
			 case 'F':
			 case 't':
			 case 'T':
			 case 'k':
			 case 'K':
			   if(sort==NO)
			   {  sort=YES;
			      prikey=*cp; /*sort by */
			      if (islower(*cp))
			      {  pridir = 1; /*lowest first*/
			      }  else
			      {  pridir = (-1); /* highest first*/
			      }
			      seckey='0'; /*default in case no sec key*/
			      secdir=1;
			   }
			   else
			   {  seckey=*cp;
			      if (islower(*cp))
			      {  secdir = 1; /*lowest first*/
			      }  else
			      {  secdir = (-1); /* highest first*/
			      }
			   }
			   break;
			 default:
			   printuse(6);
			}
		     }  /* end of for loop--options ready */
	       }
	       else
	       {  printuse(4);
	       }
	}       /* End of case statement--command line processed*/

	/*  Now on to the main program */

	rcount = 0;
	strcat(source,filenam);
	if ( (hsource = open(source, O_BINARY|O_RDWR)) == -1)
	   {  fprintf(stderr,"Couldn't open file %s",source);
	      exit(errno);
	   }

	rcount = read( hsource, troop, sizeof(troop));
	if (rcount <= 0)
	   {  fprintf(stderr,"Read failure rcount= %i",rcount);
	      exit (5);
	   }


	/* all are read in now */
	/*  Initialize other variables */
	for (i=0;i<MAXSOLD;i++) si[i]=i;
	for (i=0;(i<MAXSOLD)&&((int)troop[i].rank>=0);i++)
	{  tu[i] = (int)(troop[i].btu+troop[i].mtu);
	   fa[i] = (int)(troop[i].bfa+troop[i].mfa);
	   rx[i] = (int)(troop[i].breaction+troop[i].mreaction);
	   str[i] = (int)(troop[i].bstr+troop[i].mstr);
	   if(sort==YES)
	   {  switch (tolower(prikey))
	      {  case 'r':
		   pri[i]=pridir*(int)troop[i].rank;
		   break;
		 case 'f':
		   pri[i]=pridir*fa[i];
		   break;
		 case 't':
		   pri[i]=pridir*tu[i];
		   break;
		 case 'k':
		   if(pridir>0)
		   {  pri[i]=(int)troop[i].bpsiskill;
		   }
		   else
		   {  pri[i]=((int)troop[i].bpsiskill > 0) ?
			      (int)troop[i].bpsiskill*pridir : -255;
		   }
		   break;
		 case 'p':
		   if((int)troop[i].bpsiskill>0)
		   {  pri[i]=pridir*(int)troop[i].bpsistr;
		   }
		   else
		   {  pri[i]=(pridir > 0) ? 0 : -255;
		   }
		   break;
		 default:
		   printuse(47);
	      }
	      switch (tolower(seckey))
	      {  case 'r':
		   sec[i]=secdir*(int)troop[i].rank;
		   break;
		 case 'f':
		   sec[i]=secdir*fa[i];
		   break;
		 case 't':
		   sec[i]=secdir*tu[i];
		   break;
		 case 'p':
		   if(secdir>0)
		   {  sec[i]=(int)troop[i].bpsiskill;
		   }
		   else
		   {  sec[i]=((int)troop[i].bpsiskill > 0) ?
			      (int)troop[i].bpsiskill*secdir : -255;
		   }
		   break;
		 case '0':  /*No secondary sort specified*/
		   sec[i]=0;
		   break;
		 default:
		   printuse(48);
	      }  /*end switch*/
	   }
	   else  /*no sort, check for manual sort*/
	   {  if (manual==YES)
	      {  pri[i]=0;
		 sec[i]=0;
	      }
	   }  /*end if*/
	}  /*end for*/
	/*Correct for manual operation*/
	if (manual==YES) sort=YES;
	/* Now sort by base and ship */
	do
	{  flip=0;
	   for(i=0;(i<MAXSOLD-1)&&((int)troop[si[i+1]].rank>=0);i++)
	   {  if ( (troop[si[i+1]].base<troop[si[i]].base)
		 ||( (troop[si[i+1]].base==troop[si[i]].base)
		   &&(troop[si[i+1]].ship<troop[si[i]].ship)
		   )
		 )
	      {  flip=1;
		 temp=si[i];
		 si[i]=si[i+1];
		 si[i+1]=temp;
	      }
	      else
	      { /* Nothing*/
	      }
	   }
	} while(flip==1);

	/* Now si provides a sorted list to the troops */
	/* Now do the desired real sort of soldiers */
	if (sort==YES)
	{  oldship=(int)troop[si[0]].ship;
	   oldbase=(int)troop[si[0]].base;
	   index=0;
	   while ((index<MAXSOLD)&&((int)troop[si[index]].rank>=0))
	   {  for(i=index;((int)troop[si[i]].ship==oldship)
		  &&((int)troop[si[i]].base==oldbase)
		  &&((int)troop[si[i]].rank>=0)
		  &&(i<MAXSOLD);i++);
	      shipsort(index,i);
	      if(manual==YES && oldship!=-1)
	      {  do
		 {  showtroop(index,i,YES);
		    printf("n next screen, s to switch two soldiers");
		    ch=getch();
		    if(ch=='s')
		    {  printf("\nRow numbers to switch (first,second):");
		       temp=scanf("%i,%i",&a,&b);
		       if( (temp==2) && (index<=a && a<=i)
			   && (index<=b && b<=i))
			 sswitch(a,b);
		    }
		 } while(ch!='n');

	      }
	      index=i;
	      oldship=(int)troop[si[index]].ship;
	      oldbase=(int)troop[si[index]].base;
	   }
	}

	/* showsoldiers */
	if (manual==NO) showtroop(0,MAXSOLD,NO);

	/*Now write out the changed file to disk*/
	tlong=lseek(hsource,0L,SEEK_SET);
	rcount = write( hsource, troop, sizeof(troop));
	if (rcount <= 0)
	   {  fprintf(stderr,"Write failure rcount= %i",rcount);
	      exit (9);
	   }

	exit(0);
}

void printuse(int exnum)
{  printf("Use:  yaxe /options game_number\n");
   printf("where any number of the following can be specified:\n");
   printf("option r,R = sort by rank\n");
   printf("option f,F = sort by firing acc\n");
   printf("option t,T = sort by turn units\n");
   printf("option k,K = sort by psi skill\n");
   printf("option p,P = sort by psi strength\n");
   printf("option m allows manual adjustment of ship crews, after any ");
   printf("of the above\n");
   printf("\nThe lower case option sorts from lowest to highest, ");
   printf("while the upper case \nsorts from highest to lowest.  ");
   printf("Sorting by psi always puts the unknowns first.\n");
   printf("The first option is the primary sort key.  Second key ");
   printf("breaks ties.\n\n");
   printf("Example:  yaxe /pT 4  \n");
   printf("Will sort Game 4 to put troops with no psi training in the front");
   printf(" and the best\npsi skills in the back.  ");
   printf("If two soldiers have the same psi skill (or unknown)\n");
   printf("the highest TU one will be in front.\n\n");
   printf("To output to a file, use yaxe /pT 4 >sold.txt ");
   printf("and then print sold.txt\n");
   printf("If no options set, will display all soldiers by base & ship.");
   printf("\n");
   exit(exnum);
}

void shipsort(int start, int end)
{  int flip, i, ch;
   end--;
   do
   {  flip = 0;
      for (i=start;i<end;i++)
      {  if (pri[si[i+1]]<pri[si[i]]
	    || (pri[si[i+1]]==pri[si[i]]
	       && sec[si[i+1]]<sec[si[i]]) )
	 {  flip=1;
	    sswitch(i,i+1);
	 }
      }
   } while(flip==1);
}

void sswitch(int i, int j)
{  struct soldier tsold;
   int ttu,tfa,trx,tstr,tpri,tsec, tsi;  /*temp vars for exchange*/

   /*Backup vars in i */
   tsold=troop[si[i]];
   ttu=tu[si[i]];
   tfa=fa[si[i]];
   trx=rx[si[i]];
   tstr=str[si[i]];
   tpri=pri[si[i]];
   tsec=sec[si[i]];

   /*Now copy j to i */
   troop[si[i]]=troop[si[j]];
   tu[si[i]]=tu[si[j]];
   fa[si[i]]=fa[si[j]];
   rx[si[i]]=rx[si[j]];
   str[si[i]]=str[si[j]];
   pri[si[i]]=pri[si[j]];
   sec[si[i]]=sec[si[j]];

   /*Now put temp back in j*/
   troop[si[j]]=tsold;
   tu[si[j]]=ttu;
   fa[si[j]]=tfa;
   rx[si[j]]=trx;
   str[si[j]]=tstr;
   pri[si[j]]=tpri;
   sec[si[j]]=tsec;
}

void showtroop(int start, int end, int number)
{  int oldbase, oldship, i;

	if(number==YES) printf("   ");
	printf("Name,               RANK, TU, FA,");
	printf(" RX, STR, PS, PK, SHIP, BASE, TRNG \n");
	oldbase=(int)troop[si[0]].base;
	oldship=(int)troop[si[0]].ship;

	for(i=start;(i<end)&&((int)troop[si[i]].rank>=0);i++)
	  {  if (oldbase!=troop[si[i]].base
		|| oldship!=troop[si[i]].ship)
	     {  oldship = troop[si[i]].ship;
		oldbase = troop[si[i]].base;
		printf("\n");
	     }
	     if(number==YES) printf ("%3i ",i);

	     printf("%19s",troop[si[i]].name);
	     printf(",   %i",(int)troop[si[i]].rank);
	     printf(", %i",tu[si[i]]);
	     printf(", %i",fa[si[i]]);
	     printf(", %i",rx[si[i]]);
	     printf(", %i",str[si[i]]);
	     if ((int)troop[si[i]].bpsiskill==0)    /*no peeking!*/
	     {  printf(",       ");
	     }
	     else
	     {  /*psistr=(int)(troop[index].bpsistr+troop[index].mpsistr);*/
		printf(", %2i",(int)troop[si[i]].bpsistr);
		printf(", %2i",(int)troop[si[i]].bpsiskill);
	     }
	     printf(",   %2i,   %2i",
		(int)troop[si[i]].ship,(int)troop[si[i]].base);
	     printf(", %2i",(int)troop[si[i]].trng);

	     /* Other variables if someone figures them out
	     printf(", %2i, %2i, %2i",(int)troop[si[i]].u3,
		(int)troop[si[i]].u4,(int)troop[si[i]].u5);
	     */

	     printf("\n");
	  }
}
